﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Windows.Forms;
using HeuristicLab.Common.Resources;

namespace HeuristicLab.Visualization.ChartModes {
  public class RulerChartMode : ChartMode {
    protected Point? startPoint;

    public override Image Image {
      get { return VSImageLibrary.TrackBar; }
    }

    public override string ToolTip {
      get { return "Ruler"; }
    }

    public override Cursor Cursor {
      get { return Cursors.Cross; }
    }

    public RulerChartMode(ChartControl control) : base(control) { }

    public override void HandleOnMouseMove(object sender, MouseEventArgs e) {
      base.HandleOnMouseMove(sender, e);

      if (startPoint.HasValue) {
        try {
          chartControl.PictureBox.Refresh();
          DrawCross(startPoint.Value);
          DrawCross(e.Location);
          DrawLineBetweenLocations(startPoint.Value, e.Location);
        } finally {
          chartControl.PictureBox.Update();
        }
      }
    }

    public override void HandleOnMouseUp(object sender, MouseEventArgs e) {
      base.HandleOnMouseUp(sender, e);

      switch (e.Button) {
        case MouseButtons.Left:
          if (!startPoint.HasValue)
            startPoint = e.Location;
          else {
            if (e.Location != startPoint.Value)
              DrawInfoBox(e.Location);
            else {
              chartControl.PictureBox.Refresh();
              chartControl.PictureBox.Update();
            }

            startPoint = null;
          }
          break;
      }
    }

    protected virtual void DrawCross(Point point) {
      using (var graphics = chartControl.PictureBox.CreateGraphics())
      using (var pen = new Pen(Color.Red)) {
        graphics.DrawLine(pen, point.X - 3, point.Y - 3, point.X + 3, point.Y + 3);
        graphics.DrawLine(pen, point.X - 3, point.Y + 3, point.X + 3, point.Y - 3);
      }
    }

    protected virtual void DrawLineBetweenLocations(Point startLocation, Point endLocation) {
      using (var graphics = chartControl.PictureBox.CreateGraphics())
      using (var pen = new Pen(Color.Red))
        graphics.DrawLine(pen, startLocation.X, startLocation.Y, endLocation.X, endLocation.Y);
    }

    protected virtual void DrawInfoBox(Point location) {
      var offsetP = new Size(location.X - startPoint.Value.X, location.Y - startPoint.Value.Y);
      var offsetW = chartControl.Chart.TransformPixelToWorld(offsetP);

      double lengthP = Math.Sqrt(Math.Pow(offsetP.Width, 2) + Math.Pow(offsetP.Height, 2));
      double lengthW = Math.Sqrt(Math.Pow(offsetW.Width, 2) + Math.Pow(offsetW.Height, 2));
      string text = string.Format("Pixel: {0}{1}World: {2:##.###}", Math.Round(lengthP), Environment.NewLine, Math.Round(lengthW, 3));

      using (var graphics = chartControl.PictureBox.CreateGraphics()) {
        var textSize = graphics.MeasureString(text, SystemFonts.DefaultFont);
        int width = (int)Math.Ceiling(textSize.Width);
        int height = (int)Math.Ceiling(textSize.Height);

        graphics.FillRectangle(Brushes.White, location.X, location.Y, width, height);
        graphics.DrawRectangle(Pens.Red, location.X, location.Y, width, height);
        graphics.DrawString(text, SystemFonts.DefaultFont, Brushes.Black, location.X, location.Y + 2);
      }
    }
  }
}
